@@ -270,18 +270,20 @@ static NV_STATUS alloc_and_init_mem(uvm_gpu_t *gpu, uvm_mem_t **mem, size_t size
     *mem = NULL;
 
     if (type == MEM_ALLOC_TYPE_VIDMEM_PROTECTED) {
-        TEST_NV_CHECK_RET(uvm_mem_alloc_vidmem_protected(size, gpu, mem));
+        TEST_NV_CHECK_RET(uvm_mem_alloc_vidmem(size, gpu, mem));
         TEST_NV_CHECK_GOTO(uvm_mem_map_gpu_kernel(*mem, gpu), err);
         TEST_NV_CHECK_GOTO(ce_memset_gpu(gpu, *mem, size, 0xdead), err);
     }
     else {
-        if (type == MEM_ALLOC_TYPE_SYSMEM_DMA)
+        if (type == MEM_ALLOC_TYPE_SYSMEM_DMA) {
             TEST_NV_CHECK_RET(uvm_mem_alloc_sysmem_dma(size, gpu, NULL, mem));
-        else
+            TEST_NV_CHECK_GOTO(uvm_mem_map_gpu_kernel(*mem, gpu), err);
+        }
+        else {
             TEST_NV_CHECK_RET(uvm_mem_alloc_sysmem(size, NULL, mem));
+        }
 
         TEST_NV_CHECK_GOTO(uvm_mem_map_cpu_kernel(*mem), err);
-        TEST_NV_CHECK_GOTO(uvm_mem_map_gpu_kernel(*mem, gpu), err);
         write_range_cpu(*mem, size, 0xdeaddead);
     }
 
@@ -346,9 +348,9 @@ static NV_STATUS cpu_decrypt(uvm_channel_t *channel,
     return NV_OK;
 }
 
-// gpu_encrypt uses a secure CE for encryption (instead of SEC2). SEC2 does not
-// support encryption. The following function is copied from uvm_ce_test.c and
-// adapted to SEC2 tests.
+// gpu_encrypt uses the Copy Engine for encryption, instead of SEC2. SEC2 does
+// not support encryption. The following function is copied from uvm_ce_test.c
+// and adapted to SEC2 tests.
 static void gpu_encrypt(uvm_push_t *push,
                         uvm_mem_t *dst_mem,
                         uvm_mem_t *src_mem,
@@ -443,7 +445,6 @@ static NV_STATUS test_cpu_to_gpu_roundtrip(uvm_gpu_t *gpu, size_t copy_size, siz
     cpu_encrypt(push.channel, src_cipher, src_plain, auth_tag_mem, size, copy_size);
     gpu_decrypt(&push, dst_plain, src_cipher, auth_tag_mem, size, copy_size);
 
-
     // Wait for SEC2 before launching the CE part.
     // SEC2 is only allowed to release semaphores in unprotected sysmem,
     // and CE can only acquire semaphores in protected vidmem.
